#include <io.h>
#include <fcntl.h>
#include <stdio.h>

#include "PCap.h"

static FILE *PCapReadFilePtr = NULL;
static FILE *PCapWriteFilePtr = NULL;
static struct timeval PCapWriteTimestamp = {0,0};

int PCapRead_open( const char *filename)
{
	PCapFileHdr_t fileHdr;

	if (PCapReadFilePtr != NULL) return -1;
	PCapReadFilePtr = fopen(filename,"rb");
	if ( PCapReadFilePtr == NULL )
	{
		printf("PCapRead_open():ERROR opening file %s\n",filename);
		PCapReadFilePtr = NULL;
		return -1;
	}
	// read-off file header
	if ( fread( &fileHdr, 1, sizeof(fileHdr), PCapReadFilePtr ) < 0 )
	{
		printf("PCapRead_open():ERROR reading off header from file %s\n",filename);
		fclose(PCapReadFilePtr);
		PCapReadFilePtr = NULL;
		return -1;
	}

//	printf("PCap header:\n");
//	printf("   magic=%d (0x%08x)\n",fileHdr.magic,fileHdr.magic);
//	printf("   version_major=%d (0x%08x)\n",fileHdr.version_major,fileHdr.version_major);
//	printf("   version_minor=%d (0x%08x)\n",fileHdr.version_minor,fileHdr.version_minor);
//	printf("   thiszone=%d (0x%08x)\n",fileHdr.thiszone,fileHdr.thiszone);
//	printf("   sigfigs=%d (0x%08x)\n",fileHdr.sigfigs,fileHdr.sigfigs);
//	printf("   snaplen=%d (0x%08x)\n",fileHdr.snaplen,fileHdr.snaplen);
//	printf("   linktype=%d (0x%08x)\n",fileHdr.linktype,fileHdr.linktype);

	return 1;
}

int PCapRead_readPacket( uint8 *buf, uint32 size )
{
	PCapPktHdr_t hdr;
	int retVal;
	if (PCapReadFilePtr == NULL) return -1;

	retVal = fread( &hdr, 1, sizeof(hdr), PCapReadFilePtr );
	if ( retVal <= 0 )
	{
		return -1;
	}
	
	if (size < hdr.length)
	{
		printf("PCapRead_readPacket():ERROR:Input buffer of %u not large enough, need %u\n",
					size, hdr.length);
		return -1;
	}
	retVal = fread( buf, 1, hdr.length, PCapReadFilePtr );
	if ( (retVal < 0 ) || ( retVal != hdr.length ) )
	{
		return -1;
	}
	return hdr.length;
}


int PCapRead_close(void)
{
	int retVal;

	if (PCapReadFilePtr == NULL) return 0;

	retVal = fclose(PCapReadFilePtr);
	PCapReadFilePtr = NULL;

	return retVal;
}


int PCapWrite_open( const char *filename)
{
	PCapFileHdr_t fileHdr;

	if (PCapWriteFilePtr != NULL) return -1;
	PCapWriteFilePtr = fopen(filename,"wb");
	if ( PCapWriteFilePtr == NULL )
	{
		printf("PCapRead_open():ERROR opening file %s\n",filename);
		PCapWriteFilePtr = NULL;
		return -1;
	}

	// write a file header
	fileHdr.magic = 0xa1b2c3d4;
	fileHdr.version_major = 2;
	fileHdr.version_minor = 4;
	fileHdr.thiszone = 0;
	fileHdr.sigfigs = 0;
	fileHdr.snaplen = 65535;
	fileHdr.linktype = 1;
	if ( fwrite( &fileHdr, 1, sizeof(fileHdr), PCapWriteFilePtr ) < 0 )
	{
		printf("PCapRead_open():ERROR writing header to file %s\n",filename);
		fclose(PCapWriteFilePtr);
		PCapWriteFilePtr = NULL;
		return -1;
	}
	return 1;
}

int PCapWrite_writePacket( uint8 *buf, uint32 size )
{
	PCapPktHdr_t hdr;
	int retVal;
	if (PCapWriteFilePtr == NULL) return -1;

	hdr.timestamp = PCapWriteTimestamp;
	hdr.length = size;
	hdr.onwireLength = size;

	// increment "time" for next packet
	PCapWriteTimestamp.tv_usec++;

	retVal = fwrite( &hdr, 1, sizeof(hdr), PCapWriteFilePtr );
	if ( retVal <= 0 )
	{
		return -1;
	}

	retVal = fwrite( buf, 1, size, PCapWriteFilePtr );
	if ( (retVal < 0 ) || ( retVal != size ) )
	{
		return -1;
	}
	return size;
}

int PCapWrite_close(void)
{
	int retVal;

	if (PCapWriteFilePtr == NULL) return 0;

	retVal = fclose(PCapWriteFilePtr);
	PCapWriteFilePtr = NULL;

	return retVal;
}
