/*********************************************************************
* Copyright (c) Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*/

#ifndef _ANTS_TRIGGER_H_
#define _ANTS_TRIGGER_H_

#include "sf_av_st_mac_intf.h"
#include "co_bits.h"

// Multicast group 224.1.2.3 (ANTS demo defaults)
#define MCAST_MAC    0x01005e010203ULL
#define MCAST_IP     0xe0010203
#define MCAST_PORT   8122
#define MCAST_PROTO  17

#define PAYLOAD_BYTES_PER_WORD      32
#define PAYLOAD_BYTES_FIRST_WORD    22
#define PAYLOAD_BYTES_REST_WORDS    (PAYLOAD_BYTES_PER_WORD - PAYLOAD_BYTES_FIRST_WORD)

// ********************************************
// command and status streams
#define STREAM_COMMAND_DEPTH 	2
#define STREAM_COMMAND_WIDTH  32
#define STREAM_COMMAND_TYPE	co_uint32

#define STREAM_CMD_MAGIC_NUMBER     0xABCDEF00
#define STREAM_CMD_PING             0
#define STREAM_CMD_WREG_IPV4DEST    1
#define STREAM_CMD_WREG_IPV4PROTO   2
#define STREAM_CMD_WREG_UDPPORT     3

#define STREAM_STATUS_DEPTH 	4
#define STREAM_STATUS_WIDTH   32
#define STREAM_STATUS_TYPE	   co_uint32

// ********************************************
// pack trigger data: <msg_id[7:0]><connid[7:0]><msg_info[31:0]>
#define STREAM_TRIGGER_DEPTH  2
#define STREAM_TRIGGER_WIDTH  48
#define STREAM_TRIGGER_TYPE   co_uint48

#define STREAM_TRIGGER_MSGINFO_TYPE    co_uint32
#define STREAM_TRIGGER_CONNID_TYPE     co_uint8
#define STREAM_TRIGGER_MSGID_TYPE      co_uint8

#define STREAM_TRIGGER_MSGINFO_WIDTH   32
#define STREAM_TRIGGER_CONNID_WIDTH    8
#define STREAM_TRIGGER_MSGID_WIDTH     8

#define STREAM_TRIGGER_MSGINFO_OFFSET  0
#define STREAM_TRIGGER_CONNID_OFFSET   STREAM_TRIGGER_MSGINFO_WIDTH
#define STREAM_TRIGGER_MSGID_OFFSET    (STREAM_TRIGGER_MSGINFO_WIDTH + STREAM_TRIGGER_CONNID_WIDTH)

#define STREAM_TRIGGER_SET_FIELD(dst, val, field)  CO_BITS_PUT_U(dst, (val), STREAM_TRIGGER_##field##_OFFSET, STREAM_TRIGGER_##field##_WIDTH);
#define STREAM_TRIGGER_GET_FIELD(dst, val, field)  CO_BITS_GET_U(dst, (val), STREAM_TRIGGER_##field##_OFFSET, STREAM_TRIGGER_##field##_WIDTH);


// not all co_uint<N> are defined in co_math.h using a typedef will create an arbitrary new type
typedef uint8 co_uint269[34];

// ********************************************
// payload stream: <empty[4:0]><word_num[7:0]><data[255:0]>
#define STREAM_PAYLOAD_DEPTH  4
#define STREAM_PAYLOAD_TYPE   co_uint269
#ifdef IMPULSE_C_SYNTHESIS
#define STREAM_PAYLOAD_WIDTH  269
#else
#define STREAM_PAYLOAD_WIDTH  (sizeof(STREAM_PAYLOAD_TYPE)*8)
#endif

#define STREAM_PAYLOAD_DATA_TYPE       co_uint256
#define STREAM_PAYLOAD_INFO_TYPE       co_uint13
#define STREAM_PAYLOAD_WORDNUM_TYPE    co_uint8
#define STREAM_PAYLOAD_EMPTY_TYPE      co_uint5

#define STREAM_PAYLOAD_DATA_WIDTH      256
#define STREAM_PAYLOAD_INFO_WIDTH      13
#define STREAM_PAYLOAD_WORDNUM_WIDTH   8
#define STREAM_PAYLOAD_EMPTY_WIDTH     5

#define STREAM_PAYLOAD_DATA_OFFSET     0
#define STREAM_PAYLOAD_INFO_OFFSET     STREAM_PAYLOAD_DATA_WIDTH
#define STREAM_PAYLOAD_WORDNUM_OFFSET  STREAM_PAYLOAD_DATA_WIDTH
#define STREAM_PAYLOAD_EMPTY_OFFSET    (STREAM_PAYLOAD_DATA_WIDTH + STREAM_PAYLOAD_WORDNUM_WIDTH)

#define STREAM_PAYLOAD_SET_FIELD(dst, val, field)  CO_BITS_PUT_U(dst, (val), STREAM_PAYLOAD_##field##_OFFSET, STREAM_PAYLOAD_##field##_WIDTH);
#define STREAM_PAYLOAD_GET_FIELD(dst, val, field)  CO_BITS_GET_U(dst, (val), STREAM_PAYLOAD_##field##_OFFSET, STREAM_PAYLOAD_##field##_WIDTH);


#endif // _ANTS_TRIGGER_H_
