/*********************************************************************
* Copyright (c) Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*/

#ifdef WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <time.h>
#include "co.h"
#include "cosim_log.h"
#ifndef IMPULSE_C_TARGET
#include "ants_trigger.h"
#include "PCap.h"
#else
#include <stdlib.h>
#include "solarflare_wrapper.h"
#endif

extern co_architecture co_initialize(void *);

#define DEST_MAC_LOW 	MCAST_MAC	// lower 32-bits of MAC
#define DEST_MAC_HIGH 	(MCAST_MAC >> 32)// upper 16-bits of MAC

IF_SIM(void host_proc( co_stream cmd_strm, co_stream stat_strm, co_stream trigger_strm, co_signal start_sig))
IF_NSIM(void host_proc( co_stream cmd_strm, co_stream stat_strm, co_stream trigger_strm))
{
	STREAM_TRIGGER_TYPE trigData;
	STREAM_COMMAND_TYPE cfgData;

	printf("host_proc:Starting\n");

	// configure MAC to watch for
	co_stream_open(cmd_strm, O_WRONLY, UINT_TYPE(STREAM_COMMAND_WIDTH));
	co_stream_open(stat_strm, O_RDONLY, UINT_TYPE(STREAM_STATUS_WIDTH));
	co_stream_open(trigger_strm, O_RDONLY, UINT_TYPE(STREAM_TRIGGER_WIDTH));

   cfgData = STREAM_CMD_MAGIC_NUMBER | STREAM_CMD_PING;
   co_stream_write(cmd_strm, &cfgData, sizeof(cfgData));
   cfgData = 0x12345678;
   co_stream_write(cmd_strm, &cfgData, sizeof(cfgData)); // Dest IPv4 UDP port

   co_stream_read(stat_strm, &cfgData, sizeof(cfgData));
   printf("host_proc:Ping returned 0x%08x\n", cfgData);

	cfgData = STREAM_CMD_MAGIC_NUMBER | STREAM_CMD_WREG_IPV4DEST;
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData));
	cfgData = MCAST_IP;
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData)); // Dest IPv4

	cfgData = STREAM_CMD_MAGIC_NUMBER | STREAM_CMD_WREG_IPV4PROTO;
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData));
	cfgData = MCAST_PROTO;  // IPv4 UDP protocol
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData)); // Dest IPv4 protocol

	cfgData = STREAM_CMD_MAGIC_NUMBER | STREAM_CMD_WREG_UDPPORT;
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData));
	cfgData = MCAST_PORT;
	co_stream_write(cmd_strm, &cfgData, sizeof(cfgData)); // Dest IPv4 UDP port

	// start MAC in SW sim
	IF_SIM(co_signal_post(start_sig, 0);)

	Sleep(1500);  // sleep to let config process perform commands

	// watch for trigger(s)
	while (co_stream_read(trigger_strm, &trigData, sizeof(trigData)) == co_err_none)
	{
	   co_uint8 msg_id = 0;
	   co_uint8 conn_id = 0;
	   STREAM_TRIGGER_MSGINFO_TYPE msg_info = 0;

	   CO_BITS_GET_U(msg_id, trigData, STREAM_TRIGGER_MSGID_OFFSET, STREAM_TRIGGER_MSGID_WIDTH);
	   CO_BITS_GET_U(conn_id, trigData, STREAM_TRIGGER_CONNID_OFFSET, STREAM_TRIGGER_CONNID_WIDTH);
	   CO_BITS_GET_U(msg_info, trigData, STREAM_TRIGGER_MSGINFO_OFFSET, STREAM_TRIGGER_MSGINFO_WIDTH);

		printf("host_proc:trigger data: msg_id = %u, conn_id = %u, msg_info = %u\n", (co_uint32)msg_id, (co_uint32)conn_id, (co_uint32)msg_info);
	}

	co_stream_close(cmd_strm);
	co_stream_close(stat_strm);
	co_stream_close(trigger_strm);

	printf("host_proc:Exiting\n");
}

int main(int argc, char *argv[])
{
	co_architecture my_arch;
	void *param = NULL;

#ifndef IMPULSE_C_TARGET
	if ( argc < 3 )
	{
		printf("main():Need input and output PCap filenames\n");
		printf("main():Usage: %s <input_pcap_filename> <output_pcap_filename>\n", argv[0]);
		exit(0);
	}
	printf("main():Opening %s for input.\n", argv[1]);
	if ( PCapRead_open(argv[1]) < 0 )
	{
		printf("main():PCapRead_open() returned error, exiting.\n");
		exit(-1);
	}

	printf("main():Opening %s for output.\n", argv[2]);
	if ( PCapWrite_open(argv[2]) < 0 )
	{
		printf("main():PCapWrite_open() returned error, exiting.\n");
		exit(-1);
	}
#else
	// Initialize AOE
	Sf_Init("/dev/aoe0");
#endif

	printf("Impulse C is Copyright(c) Impulse Accelerated Technologies, Inc.\n");

	// Running on hardware
	my_arch = co_initialize(param);
	co_execute(my_arch);

#ifndef IMPULSE_C_TARGET
	PCapRead_close();
	PCapWrite_close();
#else
	// Shutdown AOE
	Sf_Close();
#endif

	printf("\n\nApplication complete.\n");// Press the Enter key to continue.\n");
	//c = getc(stdin);

	return(0);
}

