/******************************************************************************
* Copyright (c) 2008, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
* www.ImpulseC.com
*
* This source file may be used and distributed without restriction provided 
* that this copyright notice is not removed from the file and that any 
* derivative work contains this copyright notice.
*
* co_bits.h: Bit manipulation routines
*
******************************************************************************/

#ifndef _CO_BITS_H_
#define _CO_BITS_H_

#if defined(IMPULSE_C_SYNTHESIS)
/******************************************************************************
 * calls for HDL library
******************************************************************************/

typedef co_uint8 co_uint257[17];
//#define MAKE_ONES_MASK256(n) (((co_uint257)1<<(n)) - 1 )
#define MAKE_ONES_MASK256(n) (((co_uint256)-1)>>(256-(n)))
#define U256_ALL_ONES	MAKE_ONES_MASK256(256)
#define U256_MASK_START_LEN(start,len)	( (~(MAKE_ONES_MASK256(start))) & (MAKE_ONES_MASK256((start) + (len))) )

//#define CO_BITS_GET_U256(dst, src, start, len)	dst = ((src) >> (start)) & ( U256_ALL_ONES >> (len))
//#define CO_BITS_PUT_U256(dst, src, start, len)	dst = (co_uint256)((~(U256_MASK_START_LEN(start,len))) & dst) | ((co_uint256)(src & (MAKE_ONES_MASK256(len))) << (start))

#undef co_bit_extract
#undef co_bit_insert
#define CO_BITS_GET_U(dst, src, start, len)	dst = co_bit_extract(src,start,len)
#define CO_BITS_PUT_U(dst, src, start, len)	dst = co_bit_insert(dst,start,len,src)

#define COPY_VERYLONG_WORD(dst,src) dst = src;
#define ZERO_VERYLONG_WORD(x) x = 0;

#else

//memset(&dst,0,sizeof(dst));
#define ZERO_VERYLONG_WORD(x) memset(&x,0,sizeof(x));
#define COPY_VERYLONG_WORD(dst,src) ZERO_VERYLONG_WORD(dst); memcpy(&dst, &src, (sizeof(dst) > sizeof(src)) ? sizeof(src) : sizeof(dst));

/******************************************************************************
 * calls for software simulation
 ******************************************************************************/
#include <stdio.h>
#define DBG_CO_BITS_GET_U		0
#define DBG_CO_BITS_PUT_U		0

typedef unsigned char UBYTE;
typedef unsigned short UWORD;

static void co_bits_get_u( UBYTE *dst, unsigned dst_sz, UBYTE *src, unsigned src_sz, unsigned start, unsigned len)
{
	unsigned int bitOffset = start % 8;
	unsigned int curSrcByte = 0;
	unsigned int curDstByte = 0;
	int gotLastBits = 0;
	UBYTE tmpByte = 0;
	if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():dst_sz = %d\n", dst_sz);
	if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():src_sz = %d\n", src_sz);
	if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():start = %d\n", start);
	if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():len = %d\n", len);
	if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():bitOffset = %d\n", bitOffset);
	if ( len > dst_sz*8 ) fprintf(stderr,"co_bits_get_u():WARNING:Bit slice %d is greater than destination size %d\n", len, dst_sz*8);
	while ( (curSrcByte < src_sz) && (curDstByte < dst_sz) ) {
		unsigned int curSrcMaxBit = ((curSrcByte + 1)*8 - 1);
		if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():curSrcByte = %d\n", curSrcByte);
		if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():curSrcMaxBit = %d\n", curSrcMaxBit);
		if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():curDstByte = %d\n", curDstByte);
		if (curSrcMaxBit >= start) {
			if ( gotLastBits ) {
				// always zero-out top most bits 
				tmpByte = 0;
			}
			else {
            tmpByte = (src[curSrcByte] >> bitOffset);// & (((UWORD)1 << (8 - bitOffset)) - 1);
            if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():0:tmpByte = %02x\n", tmpByte);
				// stay inbounds of src, effectively zero-pads left most bits
				if ( (curSrcByte + 1 ) < src_sz ) {
               tmpByte |= (src[curSrcByte + 1] << (8 - bitOffset));// & ~(((UWORD)1 << (8 - bitOffset)) - 1);
				}
            if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():00:tmpByte = %02x\n", tmpByte);

				// handle last bits accordingly
				if ( curSrcMaxBit >= (start + len - 1)) {
					if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():last bits\n");
					tmpByte &= (((UWORD)1<<(8 - ((curSrcMaxBit - (start + len - 1)) % 8) )) - 1 );
					gotLastBits = 1;
				}
			}
			if (DBG_CO_BITS_GET_U) printf("co_bits_get_u():tmpByte = %02x\n", tmpByte);
			dst[curDstByte] = tmpByte; 
			curDstByte++;
		}
		curSrcByte++;
	}
}
static void co_bits_put_u( UBYTE *dst, unsigned dst_sz, UBYTE *src, unsigned src_sz, unsigned start, unsigned len)
{
	unsigned int bitOffset = start % 8;
	unsigned int lastBitOffset = (start + len) % 8;
	unsigned int curSrcByte = 0;
	unsigned int curDstByte = 0;

	UBYTE tmpByte;
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():dst_sz = %d\n", dst_sz);
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():src_sz = %d\n", src_sz);
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():start = %d\n", start);
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():len = %d\n", len);
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():bitOffset = %d\n", bitOffset);
	if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():lastBitOffset = %d\n", lastBitOffset);
	if ( len > dst_sz*8 ) fprintf(stderr,"co_bits_put_u():WARNING:Bit slice %d is greater than destination size %d\n", len, dst_sz*8);
	while ( (curSrcByte < src_sz) && (curDstByte < dst_sz) ) {
		unsigned int curDstMaxBit = ((curDstByte + 1)*8 - 1);
		if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():curDstByte = %d\n", curDstByte);
		if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():curSrcByte = %d\n", curSrcByte);
		if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():curDstMaxBit = %d\n", curDstMaxBit);
//		if (( curDstMaxBit >= start ) && ( curDstMaxBit <= (start + len - 1) )) {
		if (( curDstMaxBit >= start )) {
			if ( (curDstMaxBit - start) <= 8 ) {
				// first byte, lower bits are from dst, upper bits from src
				if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():first byte\n");
				tmpByte = dst[curDstByte] & (((UWORD)1<<(bitOffset)) - 1 );
				tmpByte |= (src[curSrcByte] << bitOffset) & ~(((UWORD)1<<bitOffset) - 1 );
			}
			else if ( ((start + len) - curDstMaxBit) > 8 ) {
				// middle byte, lower bits are from previous src location, upper bits from current src location
				if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():middle byte\n");
				tmpByte = (src[curSrcByte - 1] >> bitOffset) & ~(((UWORD)1<<(8-bitOffset)) - 1 );
				tmpByte |= (src[curSrcByte] << bitOffset) & (((UWORD)1<<(8-bitOffset)) - 1 );
			}
			else if ( ((start + len) - curDstMaxBit) <= 8 ) {
				// last byte, lower bits are from src, upper bits from dst
				if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():last byte\n");
				tmpByte = (src[curSrcByte] >> bitOffset) & (((UWORD)1<<(8-lastBitOffset)) - 1 );
				tmpByte |= dst[curDstByte] & ~(((UWORD)1<<(8-lastBitOffset)) - 1 );
			}
			if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():tmpByte = %02x\n", tmpByte);
			dst[curDstByte] = tmpByte; 
			curSrcByte++;
		}
      else
      {
         if (DBG_CO_BITS_PUT_U) printf("co_bits_put_u():out of bounds byte\n");
      }
		curDstByte++;
	}
}

#define CO_BITS_GET_U(dst, src, start, len)  co_bits_get_u( (UBYTE *)&(dst), sizeof(dst), (UBYTE *)&(src), sizeof(src), (start), (len) )
#define CO_BITS_PUT_U(dst, src, start, len)  co_bits_put_u( (UBYTE *)&(dst), sizeof(dst), (UBYTE *)&(src), sizeof(src), (start), (len) )

#endif

#endif //_CO_BITS_H_
