/*********************************************************************
* Copyright (c) Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*/

#include <stdio.h>
#include "co.h"
#include "cosim_log.h"
#include "ants_trigger.h"
#include "PCap.h"

//extern co_architecture co_initialize(void *);

void print_av_st(const char* str, uint8 sop, uint8 eop, uint8 empty,
		co_uint256* data)
{
	int i;
	uint64* ptr64 = (uint64*)data;

	printf("%s", str);
	printf("%s ", sop ? "SOP" : "   ");
	if (sop && (sop != 1))
	{
	   printf("(0x%02x)", sop);
	}
	printf("%s ", eop ? "EOP" : "   ");
   if (eop && (eop != 1))
   {
      printf("(0x%02x)", eop);
   }
	printf("EMPTY=0x%01x ", empty);

	printf("DATA=0x");
	for ( i = 0; i < 4; i++)
	{
		printf("%016llx ", *(ptr64 + (4 - 1) - i));
	}
	printf("\n");
}

void byteSwap256(co_uint256* in, co_uint256* out)
{
	int i;
	uint8* inPtr = ((uint8*)in) + (sizeof(*in) - 1);
	uint8* outPtr = (uint8*)out;

	for(i=0;i<32;i++)
	{
		*(outPtr++) = *(inPtr--);
	}
}

// Assumes that PCapRead_open() has already been called by main()
void ProducerMac(co_stream from_mac, co_signal start_sig)
{
	int32 pktCount = 0;
	uint8 pkt[2048];
	int i;
	int32 sigVal;

	printf("ProducerMac:Starting, waiting for start signal\n");
	co_signal_wait(start_sig, &sigVal);

	printf("ProducerMac:Writing Enet packets to stream:\n");

	co_stream_open(from_mac, O_WRONLY, UINT_TYPE(STREAM_RAW_MAC_WIDTH));

	while (1)
	{
		int pktSize;
		int wordLoopMax;
		int lastWordEmptyCount;
		AV_ST_DATA_TYPE *bufPtr;
	   AV_ST_DATA_TYPE data;
	   AV_ST_EOP_TYPE eop;
	   AV_ST_SOP_TYPE sop;
	   AV_ST_EMPTY_TYPE empty;
	// AV_ST_ERROR_TYPE error;

		STREAM_RAW_MAC_TYPE val;


		memset(pkt, sizeof(pkt), 0);
		pktSize = PCapRead_readPacket( pkt, sizeof(pkt));
		if ( pktSize < 1 ) break;

		// data is 64-bit wide network-endian
		bufPtr = (AV_ST_DATA_TYPE *)&pkt;
		wordLoopMax = (pktSize+sizeof(AV_ST_DATA_TYPE)-1)/sizeof(AV_ST_DATA_TYPE);
		lastWordEmptyCount = (sizeof(AV_ST_DATA_TYPE) - (pktSize & 0x1f)) & 0x1f;
		for ( i = 0; i < wordLoopMax; i++ )
		{
			byteSwap256(bufPtr++, &data);
			if (i == 0)
			{
				printf("ProducerMac:Sending SOP for pkt#%d\n",pktCount);
				sop = 1;
				eop = 0;
				empty = 0;
			}
			else if (i >= (wordLoopMax-1))
			{
				printf("ProducerMac:Sending EOP for pkt#%d\n",pktCount);
				sop = 0;
				eop = 1;
				empty = lastWordEmptyCount;
			}
			else
			{
				sop = 0;
				eop = 0;
				empty = 0;
			}
         AV_ST_SET_FIELD(val, data, DATA);
         AV_ST_SET_FIELD(val, sop, SOP);
         AV_ST_SET_FIELD(val, eop, EOP);
         AV_ST_SET_FIELD(val, empty, EMPTY);

			print_av_st("ProducerMac:Write ",sop,eop,empty,&data);
			co_stream_write(from_mac, &val, sizeof(val));
		}

		pktCount++;
	}

	printf("\n");

	printf("ProducerMac:Total packets written to stream = %d\n", pktCount);
	// signal end of SW simulation (EOS not supported in HW for raw stream)
	co_stream_close(from_mac);

	printf("ProducerMac:Exiting\n");
}

// Assumes that WriteCap_open() has already been called by main()
void ConsumerMac(co_stream to_mac)
{
	STREAM_RAW_MAC_TYPE inValue;
	uint8 pkt[2048];
	AV_ST_DATA_TYPE *ptrData = (AV_ST_DATA_TYPE*)pkt;
	int inCount = 0;
	int pktCount = 0;

	AV_ST_DATA_TYPE data;
	AV_ST_EOP_TYPE eop;
	AV_ST_SOP_TYPE sop;
	AV_ST_EMPTY_TYPE empty;
//	AV_ST_ERROR_TYPE error;

	printf("ConsumerMac:Starting\n");

	co_stream_open(to_mac, O_RDONLY, UINT_TYPE(STREAM_RAW_MAC_WIDTH));

	// EOS is end of SW sim
	while (co_stream_read(to_mac, &inValue, sizeof(inValue)) == co_err_none)
	{
      AV_ST_GET_FIELD(data, inValue, DATA);
      AV_ST_GET_FIELD(eop, inValue, EOP);
      AV_ST_GET_FIELD(sop, inValue, SOP);
      AV_ST_GET_FIELD(empty, inValue, EMPTY);
//       AV_ST_GET_FIELD(error, inValue, ERROR);

		print_av_st("ConsumerMac:Read from src ",sop,eop,empty,&data);

		if (sop)
		{
			printf("ConsumerMac:Received SOP for pkt#%d\n",pktCount);
			ptrData = (AV_ST_DATA_TYPE*)pkt;

			inCount = 0;
		}

		byteSwap256(&data, ptrData++);

		if (eop)
		{
			inCount += sizeof(AV_ST_DATA_TYPE) - empty;
			printf("ConsumerMac:Received EOP for pkt#%d size=%d(0x%04x)\n",pktCount, inCount, inCount);
			if (PCapWrite_writePacket(pkt, inCount) < 1)
			{
				printf("ConsumerMac:PCapWrite_writePacket returned error, exiting\n");
			}
			pktCount++;
		}
		else
		{
			inCount += sizeof(AV_ST_DATA_TYPE);
		}
	}
	co_stream_close(to_mac);

	printf("ConsumerMac:Received %d packets\n", pktCount);

	printf("ConsumerMac:Exiting\n");
}
