/**************************************************************************\
 * *    Filename: ants_reg_access.c
 * * Description: ANTS low level access routines file
 * *
 * * Copyright (c) 2015-2016, Solarflare Communications Inc.
 * * All rights reserved.
 * *
 * * Redistribution and use in source and binary forms, with or without
 * * modification, are permitted provided that the following conditions are
 * * met:
 * *
 * * * Redistributions of source code must retain the above copyright notice,
 * *   this list of conditions and the following disclaimer.
 * *
 * * * Redistributions in binary form must reproduce the above copyright
 * *   notice, this list of conditions and the following disclaimer in the
 * *   documentation and/or other materials provided with the distribution.
 * *
 * * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * \**************************************************************************/
#include "ants_reg_access.h"

#ifdef WRITE_DEBUG
#undef WRITE_DEBUG
#endif

amm_err_t ants_write_reg_block(amm_handle_t *handle, uint32_t  offset, uint32_t length, uint8_t  *data)
{

  uint32_t loop_count = length / SF_HW_BLOCK_REG_WIDTH;
  uint32_t block_len = SF_HW_BLOCK_REG_WIDTH;
  uint32_t idx;
  amm_err_t ret = AMM_SUCCESS;

  for (idx = 0; idx < loop_count; idx++) {

#ifdef WRITE_DEBUG
    ANTS_LOG_DEBUG("Writing 0x%08X to 0x%016"PRIX64,
            *(uint32_t*)(((uint8_t *) data) + (idx * SF_HW_BLOCK_REG_WIDTH)),
            (offset + (idx * SF_HW_BLOCK_REG_WIDTH)));
#endif

    ret = amm_writemem(handle,
                       offset + (idx * SF_HW_BLOCK_REG_WIDTH),
                       (void*)(data + (idx * SF_HW_BLOCK_REG_WIDTH)),
                       &block_len,
                       AMM_PATH_FC);

    if (ret != AMM_SUCCESS)
      break;
  }

  return ret;
}

/**
 * @brief Writing to a particular address
 *
 * @param handle <amm_handle_t> handle to the device
 * @param address <uint32_t> address
 * @param value <uint32_t> value to write on the supplied address
 *
 * @return AMM_SUCCESS or different error codes
 */
amm_err_t ants_write_reg(amm_handle_t handle,uint32_t address, uint32_t value)
{
  uint32_t length = SF_HW_BLOCK_REG_WIDTH;
  amm_err_t status = AMM_SUCCESS;

#ifdef WRITE_DEBUG
  ANTS_LOG_DEBUG("Writing 0x%08X to 0x%x" PRIX64, value, address);
#endif

  status = amm_writemem(handle, address, (void *)&value, &length, AMM_PATH_FC);

  return status;
}

/**
 * @brief Reading from a particular address
 *
 * @param handle <amm_handle_t> handle to the device
 * @param address <uint32_t> address
 * @param value <uint32_t> pointer to store the value on the address
 *
 * @return AMM_SUCCESS or different error codes
 */
amm_err_t ants_read_reg (amm_handle_t handle, uint32_t address, uint32_t *value)
{
  uint32_t length = SF_HW_BLOCK_REG_WIDTH;
  amm_err_t status;

  /* Parameter Validation */
  if(NULL == handle)
    return AMM_ERROR_NOT_OPEN;

  status = amm_readmem(handle, (void *)value, address, &length, AMM_PATH_FC);

  return status;
}
