/*********************************************************************
* Copyright (c) 2012, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co.h: Impulse C interface for the Altera Nios II platform.
*
* Definitions of Impulse C functions, including:
*   - process creation and execution
*   - bit operations
*   - streams
*   - signals
*   - registers
*   - shared memory
*
*********************************************************************/

#ifndef CO_H
#define CO_H

#include <stddef.h>

/* 32-bit Nios II */
#define IMPULSE_C_TARGET NIOS_II_32
#define IMPULSE_C_BUS AVALON
#define REGISTER_WIDTH 32

#include "co_if_sim.h"
#include "co_types.h"

extern co_process co_process_create(const char * name, co_function run, int argc, ...);
extern void co_execute(co_architecture arch);
#define cosim_architecture_config(arch, attribute, value)

/* The datatype used in stream operations is the register width of the
 * architecture, so 'width' is effectively ignored. */
extern co_type co_type_create(co_sort sort, unsigned int width);
#define INT_TYPE(width)		co_type_create(co_int_sort,width)
#define UINT_TYPE(width) 	co_type_create(co_uint_sort,width)
#define CHAR_TYPE			co_type_create(co_int_sort,8)
#define FLOAT_TYPE			co_type_create(co_float_sort,32)
#define DOUBLE_TYPE			co_type_create(co_float_sort,64)

#define co_bit_extract(value, start_bit, num_bits) (int32)(((value) >> (start_bit)) & (~(0xffffffffU << (num_bits))))
#define co_bit_extract_u(value, start_bit, num_bits) (uint32)((start_bit) >= 32 ? 0 : ((value) >> (start_bit)) & (~(0xffffffffU << (num_bits))))
#define co_bit_insert(destination, dest_start_bit, num_bits, source) \
	(int32)( \
	    ( (destination) & ( 0xffffffffU << ( (dest_start_bit) + (num_bits) ) ) ) | \
        ( ( (source) << (dest_start_bit) ) & ~( ( 0xffffffffU << ( (dest_start_bit) + (num_bits) ) ) | ~( 0xffffffffU << (dest_start_bit) ) ) ) | \
        ( (destination) & ~( 0xffffffffU << (dest_start_bit) ) ) \
	)
#define co_bit_insert_u(destination, dest_start_bit, num_bits, source) (uint32)co_bit_insert((int32)(destination), (dest_start_bit), (num_bits), (int32)(source))

/* 'bufferdepth' and 'datatype' are ignored when creating a stream. */
extern co_stream co_stream_create(const char * name, co_type datatype, unsigned int bufferdepth);
extern void co_stream_attach(co_stream stream, void * io, co_stream_direction dir);
extern co_error co_stream_write(co_stream stream, const void * buffer, unsigned int buffersize);
extern co_error co_stream_read(co_stream stream, void * buffer, unsigned int buffersize);
extern int co_stream_read_nb(co_stream stream, void * buffer, unsigned int buffersize);
extern co_error co_stream_close(co_stream stream);
extern co_error co_stream_free(co_stream stream);

#define co_stream_open(stream,mode,type)
#define co_stream_eos(stream) (((nr_impc_status((stream)->io)&2)!=0) || (nr_impc_status((stream)->io)==0))

/* Assembler function prototypes -- do not call! */
extern void nr_impc_write(int var, void * io);
extern int nr_impc_read(int * var_addr, void * io);
extern int nr_impc_read_nb(int * var_addr, void * io);
extern void nr_impc_close(void * io);
extern int nr_impc_status(void * io);

#define HW_STREAM_OPEN(proc,stream,mode,type)

#define HW_STREAM_WRITE(proc,stream,var) \
  nr_impc_write(var,(stream)->io)

#define HW_STREAM_READ(proc,stream,var,evar) \
  (evar=nr_impc_read(&(var),(stream)->io))
  
#define HW_STREAM_READ_NB(proc,stream,var,evar) \
  (!(evar=(!nr_impc_read_nb(&(var),(stream)->io))))

#define HW_STREAM_CLOSE(proc,stream) \
  if (stream->direction==HW_INPUT) nr_impc_close((stream)->io)

extern co_signal co_signal_create(const char * name);
extern void co_signal_attach(co_signal signal, void * io, co_stream_direction dir);
extern co_error co_signal_post(co_signal signal, int32 value);
extern co_error co_signal_wait(co_signal signal, int32 * vp);

extern co_register co_register_create(const char * name, co_type type);
void co_register_attach(co_register reg, void * io, co_stream_direction dir);
extern co_error co_register_write(co_register reg, const void * buffer, unsigned int buffersize);
extern co_error co_register_read(co_register reg, void * buffer, unsigned int buffersize);
extern void co_register_put(co_register reg, int32 value);
extern int32 co_register_get(co_register reg);

extern co_memory co_memory_create(const char * name, const char * loc, unsigned int size, void *(*alloc)(size_t));
extern co_error co_memory_writeblock(co_memory mem, unsigned int offset, void * buf, unsigned int buffersize);
extern co_error co_memory_readblock(co_memory mem, unsigned int offset, void * buf, unsigned int buffersize);
extern void * co_memory_ptr(co_memory mem);

#ifndef IMPULSE_C_SYNTHESIS
#define co_par_break()
#endif

#include "co_math.h"

#endif  /* CO_H */

