/*********************************************************************
* Copyright (c) 2012, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_memory.c: Functions for creating, reading, and writing memories.
*
*********************************************************************/

#include "system.h"

#include <malloc.h>
#include <string.h>

#include "co.h"

co_memory co_memory_create(const char * name, const char * loc, unsigned int size, void *(*alloc)(size_t))
{
    co_memory m;

	m = (co_memory) malloc(sizeof(co_memory_t));
	if ( m != NULL ) {
		m->name = (name != NULL) ? strdup(name) : NULL;
		m->data = (*alloc)(size);
		m->size = size;
	}
  
	return(m);
}

co_error co_memory_readblock(co_memory m, unsigned int offset, void * buf, unsigned int bufsize)
{
	if ( m == NULL || (offset + bufsize) > m->size ) {
		return co_err_invalid_arg;
	}

	memcpy(buf, ((char *) m->data) + offset, bufsize);
	
	return co_err_none;
}

co_error co_memory_writeblock(co_memory m, unsigned int offset, void * buf, unsigned int bufsize)
{
	if ( m == NULL || (offset + bufsize) > m->size ) {
		return co_err_invalid_arg;
	}

	memcpy(((char *) m->data) + offset, buf, bufsize);

	return co_err_none;
}

void * co_memory_ptr(co_memory m)
{
    return (m != NULL) ? m->data : NULL;
}

