-- 
-- Copyright (c) 2012 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
--

library ieee;
use ieee.std_logic_1164.all;

use work.avalon_if.all;

entity ants_trigger_arch_module is
	generic (
		INCLUDE_HEADER_BLOCK : integer := 0
	);
	port (
		reset : in std_logic;
		clk : in std_logic;
--		co_reset : in std_logic;
--		co_clk : in std_logic;

		-- Avalon-ST Packet Connection mac_in_strm_port for bustype avalon_st_raw
		mac_in_strm_port_ready : out std_logic;
		mac_in_strm_port_valid : in std_logic;
		mac_in_strm_port_data : in std_logic_vector (263-1 downto 0);

		-- Avalon-ST Packet Connection mac_out_strm_port for bustype avalon_st_raw
		mac_out_strm_port_ready : in std_logic;
		mac_out_strm_port_valid : out std_logic;
		mac_out_strm_port_data : out std_logic_vector (263-1 downto 0);

		-- Avalon-ST Packet Connection command_strm_port for bustype avalon_st_raw
		command_strm_port_ready : out std_logic;
		command_strm_port_valid : in std_logic;
		command_strm_port_data : in std_logic_vector (32-1 downto 0);

		-- Avalon-ST Packet Connection status_strm_port for bustype avalon_st_raw
		status_strm_port_ready : in std_logic;
		status_strm_port_valid : out std_logic;
		status_strm_port_data : out std_logic_vector (32-1 downto 0);

		-- Avalon-ST Packet Connection trigger_strm_port for bustype avalon_st_raw
		trigger_strm_port_ready : in std_logic;
		trigger_strm_port_valid : out std_logic;
		trigger_strm_port_data : out std_logic_vector (48-1 downto 0);
		-- header_block slave port
		header_block_ce : in std_logic;
		header_block_read_n : in std_logic;
		header_block_write_n : in std_logic;
		header_block_addr : in std_logic_vector (1 downto 0);
		header_block_wdata : in std_logic_vector (63 downto 0);
		header_block_rdata : out std_logic_vector (63 downto 0)
	);
end;

architecture impl of ants_trigger_arch_module is

	component ants_trigger_arch_header_block_module is
	  port (
		 reset : in std_ulogic;
		 clk : in std_ulogic;
		 avalon_ce : in std_logic;
		 avalon_read_n : in std_logic;
		 avalon_write_n : in std_logic;
		 avalon_addr : in std_logic_vector (1 downto 0);
		 avalon_wdata : in std_logic_vector (63 downto 0);
		 avalon_rdata : out std_logic_vector (63 downto 0)
		 );
	end component;
  component ants_trigger_arch is
    port (
    reset : in std_ulogic;
    sclk : in std_ulogic;
    clk : in std_ulogic;
    mac_in_strm_port_en : in std_ulogic;
    mac_in_strm_port_eos : in std_ulogic;
    mac_in_strm_port_data : in std_ulogic_vector (262 downto 0);
    mac_in_strm_port_rdy : out std_ulogic;
    mac_out_strm_port_en : in std_ulogic;
    mac_out_strm_port_data : out std_ulogic_vector (262 downto 0);
    mac_out_strm_port_eos : out std_ulogic;
    mac_out_strm_port_rdy : out std_ulogic;
    command_strm_port_en : in std_ulogic;
    command_strm_port_eos : in std_ulogic;
    command_strm_port_data : in std_ulogic_vector (31 downto 0);
    command_strm_port_rdy : out std_ulogic;
    status_strm_port_en : in std_ulogic;
    status_strm_port_data : out std_ulogic_vector (31 downto 0);
    status_strm_port_eos : out std_ulogic;
    status_strm_port_rdy : out std_ulogic;
    trigger_strm_port_en : in std_ulogic;
    trigger_strm_port_data : out std_ulogic_vector (47 downto 0);
    trigger_strm_port_eos : out std_ulogic;
    trigger_strm_port_rdy : out std_ulogic
    );
  end component;

  signal mac_in_strm_port_rdy : std_ulogic;
  signal mac_in_strm_port_en : std_ulogic;
  signal mac_in_strm_port_eos : std_ulogic;
  signal mac_in_strm_port_idata : std_ulogic_vector (262 downto 0);
  signal mac_out_strm_port_rdy : std_ulogic;
  signal mac_out_strm_port_en : std_ulogic;
  signal mac_out_strm_port_eos : std_ulogic;
  signal mac_out_strm_port_idata : std_ulogic_vector (262 downto 0);
  signal command_strm_port_rdy : std_ulogic;
  signal command_strm_port_en : std_ulogic;
  signal command_strm_port_eos : std_ulogic;
  signal command_strm_port_idata : std_ulogic_vector (31 downto 0);
  signal status_strm_port_rdy : std_ulogic;
  signal status_strm_port_en : std_ulogic;
  signal status_strm_port_eos : std_ulogic;
  signal status_strm_port_idata : std_ulogic_vector (31 downto 0);
  signal trigger_strm_port_rdy : std_ulogic;
  signal trigger_strm_port_en : std_ulogic;
  signal trigger_strm_port_eos : std_ulogic;
  signal trigger_strm_port_idata : std_ulogic_vector (47 downto 0);
  signal combined_reset : std_logic;
  signal reset_u_n : std_ulogic;
  signal co_reset : std_ulogic;
  signal co_clk : std_ulogic;
begin
	reset_u_n <= std_ulogic(not reset);
	co_reset <= std_ulogic(reset);
	co_clk <= std_ulogic(clk);
	

	gen_header_block: if INCLUDE_HEADER_BLOCK > 0 generate
	ants_trigger_arch_header_block_module_0: ants_trigger_arch_header_block_module
		port map (
			clk => clk,
			reset => reset,
			avalon_addr => header_block_addr,
			avalon_ce => header_block_ce,
			avalon_read_n => header_block_read_n,
			avalon_write_n => header_block_write_n,
			avalon_wdata => header_block_wdata,
			avalon_rdata => header_block_rdata
		);
	end generate gen_header_block;
	gen_header_block_not: if INCLUDE_HEADER_BLOCK < 1 generate
		header_block_rdata <= (Others => '0');
	end generate gen_header_block_not;

-- BEGIN:generateTopModuleVhdlInstances 
	mac_in_strm_port_if: avalon_st_to_no_eos_stream
		generic map (
				datawidth => 263
			)
			port map (
				clk => clk,
				reset => reset,
				avalon_valid => mac_in_strm_port_valid,
				avalon_ready => mac_in_strm_port_ready,
				avalon_data => mac_in_strm_port_data,
				stream_en => mac_in_strm_port_en,
				stream_rdy => mac_in_strm_port_rdy,
				stream_eos => mac_in_strm_port_eos,
				stream_data => mac_in_strm_port_idata
			);
	
	mac_out_strm_port_if: no_eos_stream_to_avalon_st
		generic map (
				datawidth => 263
			)
			port map (
				clk => clk,
				reset => reset,
				avalon_valid => mac_out_strm_port_valid,
				avalon_ready => mac_out_strm_port_ready,
				avalon_data => mac_out_strm_port_data,
				stream_en => mac_out_strm_port_en,
				stream_rdy => mac_out_strm_port_rdy,
				stream_eos => mac_out_strm_port_eos,
				stream_data => mac_out_strm_port_idata
			);
	
	command_strm_port_if: avalon_st_to_no_eos_stream
		generic map (
				datawidth => 32
			)
			port map (
				clk => clk,
				reset => reset,
				avalon_valid => command_strm_port_valid,
				avalon_ready => command_strm_port_ready,
				avalon_data => command_strm_port_data,
				stream_en => command_strm_port_en,
				stream_rdy => command_strm_port_rdy,
				stream_eos => command_strm_port_eos,
				stream_data => command_strm_port_idata
			);
	
	status_strm_port_if: no_eos_stream_to_avalon_st
		generic map (
				datawidth => 32
			)
			port map (
				clk => clk,
				reset => reset,
				avalon_valid => status_strm_port_valid,
				avalon_ready => status_strm_port_ready,
				avalon_data => status_strm_port_data,
				stream_en => status_strm_port_en,
				stream_rdy => status_strm_port_rdy,
				stream_eos => status_strm_port_eos,
				stream_data => status_strm_port_idata
			);
	
	trigger_strm_port_if: no_eos_stream_to_avalon_st
		generic map (
				datawidth => 48
			)
			port map (
				clk => clk,
				reset => reset,
				avalon_valid => trigger_strm_port_valid,
				avalon_ready => trigger_strm_port_ready,
				avalon_data => trigger_strm_port_data,
				stream_en => trigger_strm_port_en,
				stream_rdy => trigger_strm_port_rdy,
				stream_eos => trigger_strm_port_eos,
				stream_data => trigger_strm_port_idata
			);
	
-- END:generateTopModuleVhdlInstances 
	combined_reset <= reset or co_reset;

  ants_trigger_arch_0: ants_trigger_arch
    port map (
      reset => combined_reset,
      sclk => co_clk,
      clk => clk,
      mac_in_strm_port_en => mac_in_strm_port_en,
      mac_in_strm_port_eos => mac_in_strm_port_eos,
      mac_in_strm_port_data => mac_in_strm_port_idata,
      mac_in_strm_port_rdy => mac_in_strm_port_rdy,
      mac_out_strm_port_en => mac_out_strm_port_en,
      mac_out_strm_port_data => mac_out_strm_port_idata,
      mac_out_strm_port_eos => mac_out_strm_port_eos,
      mac_out_strm_port_rdy => mac_out_strm_port_rdy,
      command_strm_port_en => command_strm_port_en,
      command_strm_port_eos => command_strm_port_eos,
      command_strm_port_data => command_strm_port_idata,
      command_strm_port_rdy => command_strm_port_rdy,
      status_strm_port_en => status_strm_port_en,
      status_strm_port_data => status_strm_port_idata,
      status_strm_port_eos => status_strm_port_eos,
      status_strm_port_rdy => status_strm_port_rdy,
      trigger_strm_port_en => trigger_strm_port_en,
      trigger_strm_port_data => trigger_strm_port_idata,
      trigger_strm_port_eos => trigger_strm_port_eos,
      trigger_strm_port_rdy => trigger_strm_port_rdy
    );


end;
