-- 
-- Copyright (c) 2012 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.avalon_if.all;
entity ants_trigger_arch_header_block_module is
  port (
    reset : in std_ulogic;
    clk : in std_ulogic;
    avalon_ce : in std_logic;
    avalon_read_n : in std_logic;
    avalon_write_n : in std_logic;
    avalon_addr : in std_logic_vector (1 downto 0);
    avalon_wdata : in std_logic_vector (63 downto 0);
    avalon_rdata : out std_logic_vector (63 downto 0)
	 );
end;

architecture impl of ants_trigger_arch_header_block_module is
	constant VERDIG3		: std_logic_vector(15 downto 0) := std_logic_vector(to_unsigned(2,16));
	constant VERDIG2		: std_logic_vector(15 downto 0) := std_logic_vector(to_unsigned(0,16));
	constant VERDIG1		: std_logic_vector(15 downto 0) := std_logic_vector(to_unsigned(0,16));
	constant VERDIG0		: std_logic_vector(15 downto 0) := std_logic_vector(to_unsigned(0,16));
	constant cTIMESTAMP 	: std_logic_vector(63 downto 0) := X"2016050100193748";
	constant cIMPULSEC 	: std_logic_vector(63 downto 0) := X"496d70756c736543"; -- "ImpulseC" in hex
	constant cPSPVERSION	: std_logic_vector(63 downto 0) := VERDIG3 & VERDIG2 & VERDIG1 & VERDIG0;

	signal scratch_reg : std_logic_vector(63 downto 0);
begin

	-- r/w register to help detect and ensure reset and correct FPGA loading
	scratch_reg_proc: process (reset, clk)
	begin
		if (reset = '1') then
			scratch_reg <= X"FEEDFACECAFEBEEF";
		elsif (clk'event and clk='1') then
			if (avalon_ce = '1' and avalon_write_n = '0' and avalon_addr = "00") then
				scratch_reg <= avalon_wdata;
			end if;
		end if;
	end process;

	-- addresses are on 128-bit boundaries for simplicity, upper 64-bits is undefined
	avalon_rdata <= scratch_reg 	when avalon_addr = "00" else
						 cIMPULSEC 		when avalon_addr = "01" else
						 cPSPVERSION	when avalon_addr = "10" else
						 cTIMESTAMP		when avalon_addr = "11" else
						 X"A0A0A0A050505050";

end;
