#####
# This Tcl script generates the top module interface replacing SOPC Builder's 
#  generic generator in order to dynamically wrap the module including or excluding
#  optional Avalon-ST signals based on configuration via the GUI for the module
#####

set includeHeaderBlock [get_parameter_value INCLUDE_HEADER_BLOCK]
if { [lsearch [get_parameters] USE_INTERNAL_CO_MEMORY_CONFIG] < 0} {
	set useInternalCoMemoryConfig 0
} else {
	set useInternalCoMemoryConfig [get_parameter_value USE_INTERNAL_CO_MEMORY_CONFIG]
}
set internalCoMemoryBaseList [list ]

proc outputLinesWithNoTrailingDelimiter {fd lines indent delim} {
	set newLine ""
	foreach line $lines {
		puts -nonewline $fd "${newLine}${indent}${line}"
		set newLine "${delim}\n"
	}
	puts $fd ""
}

puts $instanceOutFile "
-- 
-- Copyright (c) 2012 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
--

library ieee;
use ieee.std_logic_1164.all;
"
#set paramPropList [get_parameter_properties]
set paramList [get_parameters]
#foreach param $paramList {
#	foreach paramProp $paramPropList {
#		send_message info "$paramProp = [get_parameter_property $param $paramProp ]"
#	}
#}

puts $instanceOutFile "
entity $outputname is
--	generic (
"
# generate generics
#lappend genericLines
#foreach param $paramList {
#	set hdlParam [get_parameter_property $param HDL_PARAMETER ]
##	puts $instanceOutFile "      -- $param : HDL_PARAMETER $hdlParam"
#	if {$hdlParam} {
#		lappend genericLines "$param : [get_parameter_property $param TYPE ] := [get_parameter_property $param DEFAULT_VALUE ]"
#	}
#}
#outputLinesWithNoTrailingDelimiter $instanceOutFile $genericLines "      " ";"
	 	 
puts $instanceOutFile "
--	);
	port (
"
# generate ports
lappend portLines
lappend portTermPortLines
set intfList [get_interfaces]
foreach intf $intfList {
#	puts $instanceOutFile "      -- Interface: $intf"
	lappend portLines "-- Interface: $intf"
	foreach port [get_interface_ports $intf] {
#		puts $instanceOutFile "       -- Port: $port"
#		foreach portProp [get_port_properties] {
#			puts $instanceOutFile "        -- PortProp $portProp = [get_port_property $port $portProp]"
#		}
		set dir [get_port_property $port DIRECTION]
		set width [get_port_property $port WIDTH]
		if { $dir == "Input" } {
			set dir "in"
		} else {
			set dir "out"
		}
		if { $width > 1 } {
			set sigType "std_logic_vector(${width}-1 downto 0)"
			set defVal "(others=>'0')"
		} else {
			set sigType "std_logic"
			set defVal "'0'"
		}
		lappend portLines "$port : $dir $sigType"
		
		set portBitsOnStream "[string toupper $port]BITSONSTREAM"
		if { [lsearch [get_parameters] $portBitsOnStream] > -1}	{
   		set bitsOnStream [get_parameter_value $portBitsOnStream]
   		if { $dir == "out" && $bitsOnStream == 0 } {
   			lappend portTermPortLines "$port <= $defVal;"
   		}
		}
	}
}
outputLinesWithNoTrailingDelimiter $instanceOutFile $portLines "      " ";"

puts $instanceOutFile "
	);
end;
"
puts $instanceOutFile "
architecture impl of $outputname is
   component ants_trigger_arch_module is
		generic (
			INCLUDE_HEADER_BLOCK : integer := 0
		);
		port (
			reset : in std_logic;
			clk : in std_logic;
--			co_reset : in std_logic;
--			co_clk : in std_logic;

		-- Avalon-ST Packet Connection mac_in_strm_port for bustype avalon_st_raw
		mac_in_strm_port_ready : out std_logic;
		mac_in_strm_port_valid : in std_logic;
		mac_in_strm_port_data : in std_logic_vector (263-1 downto 0);

		-- Avalon-ST Packet Connection mac_out_strm_port for bustype avalon_st_raw
		mac_out_strm_port_ready : in std_logic;
		mac_out_strm_port_valid : out std_logic;
		mac_out_strm_port_data : out std_logic_vector (263-1 downto 0);

		-- Avalon-ST Packet Connection command_strm_port for bustype avalon_st_raw
		command_strm_port_ready : out std_logic;
		command_strm_port_valid : in std_logic;
		command_strm_port_data : in std_logic_vector (32-1 downto 0);

		-- Avalon-ST Packet Connection status_strm_port for bustype avalon_st_raw
		status_strm_port_ready : in std_logic;
		status_strm_port_valid : out std_logic;
		status_strm_port_data : out std_logic_vector (32-1 downto 0);

		-- Avalon-ST Packet Connection trigger_strm_port for bustype avalon_st_raw
		trigger_strm_port_ready : in std_logic;
		trigger_strm_port_valid : out std_logic;
		trigger_strm_port_data : out std_logic_vector (48-1 downto 0);
			-- header_block slave port
			header_block_ce : in std_logic;
			header_block_read_n : in std_logic;
			header_block_write_n : in std_logic;
			header_block_addr : in std_logic_vector (1 downto 0);
			header_block_wdata : in std_logic_vector (63 downto 0);
			header_block_rdata : out std_logic_vector (63 downto 0)
		);
   end component;
"
puts $instanceOutFile "
		constant INCLUDE_HEADER_BLOCK : integer := $includeHeaderBlock;
		constant MAC_IN_STRM_PORT_TOTAL_STREAM_WIDTH : integer := 263;
		constant MAC_OUT_STRM_PORT_TOTAL_STREAM_WIDTH : integer := 263;
		constant COMMAND_STRM_PORT_TOTAL_STREAM_WIDTH : integer := 32;
		constant STATUS_STRM_PORT_TOTAL_STREAM_WIDTH : integer := 32;
		constant TRIGGER_STRM_PORT_TOTAL_STREAM_WIDTH : integer := 48;
		signal mac_in_strm_port_data_in : std_logic_vector(MAC_IN_STRM_PORT_TOTAL_STREAM_WIDTH-1 downto 0);
		signal mac_out_strm_port_data_out : std_logic_vector(MAC_OUT_STRM_PORT_TOTAL_STREAM_WIDTH-1 downto 0);
		signal command_strm_port_data_in : std_logic_vector(COMMAND_STRM_PORT_TOTAL_STREAM_WIDTH-1 downto 0);
		signal status_strm_port_data_out : std_logic_vector(STATUS_STRM_PORT_TOTAL_STREAM_WIDTH-1 downto 0);
		signal trigger_strm_port_data_out : std_logic_vector(TRIGGER_STRM_PORT_TOTAL_STREAM_WIDTH-1 downto 0);
"

if { $useInternalCoMemoryConfig == 1 } {
	puts $instanceOutFile "
	-- signals and constants for internal co_memory configuration
	constant CFG_STATE_START : integer := 2;  -- # cycles after reset to wait before starting
	constant CFG_STATE_END : integer := CFG_STATE_START + [llength $internalCoMemoryBaseList];  -- # last cycle ending configuration 
"

	foreach parm $internalCoMemoryBaseList {
		puts $instanceOutFile "   constant $parm : std_logic_vector(31 downto 0) := X\"[format "%08x" [get_parameter_value $parm]]\";"
	}
	
	puts $instanceOutFile "
	signal config_ce      : std_logic;
	signal config_read_n  : std_logic;
	signal config_write_n : std_logic;
	signal config_addr    : std_logic_vector (1 downto 0);
	signal config_wdata   : std_logic_vector (63 downto 0);
	signal config_rdata   : std_logic_vector (63 downto 0);
	signal config_state   : std_logic_vector (CFG_STATE_END downto 0);
"
}

puts $instanceOutFile "
begin
   -- Make data in/out connections"
# for each <stream>_STREAMTOTALBITS assemble/disassemble the data in/out 
#  in the format MSB->LSB: <error><empty><data>
foreach param [get_parameters] {
	if { [regexp {(.*)_STREAMTOTALBITS} $param match streamName] } {
		puts $instanceOutFile "   -- $streamName"
		set intfNameUpper [string toupper $streamName]
		set portBase "[string tolower $streamName]"
		set dir [get_port_property ${portBase}_data DIRECTION]
		if { $dir == "Input" } {
			set dir "in"
			set dataSig "${portBase}_data_${dir}"
			set msbLOffset "1"
			set msbROffset "0"
			if { [get_parameter_value ${intfNameUpper}_ERRORBITSONSTREAM] != 0} {
				append msbROffset "+[get_parameter_value ${intfNameUpper}_NUMERRORBITS]"
				if { [get_parameter_value ${intfNameUpper}_NUMERRORBITS] > 1} {
				puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset}) downto ${dataSig}'length-(${msbROffset})) <= ${portBase}_error;"
				} else {
					puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset})) <= ${portBase}_error;"
				}
				append msbLOffset "+[get_parameter_value ${intfNameUpper}_NUMERRORBITS]" 
			}
			if { [get_parameter_value ${intfNameUpper}_EMPTYBITSONSTREAM] != 0} {
				append msbROffset "+[get_parameter_value ${intfNameUpper}_NUMEMPTYBITS]"
				if { [get_parameter_value ${intfNameUpper}_NUMEMPTYBITS] > 1} {
				puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset}) downto ${dataSig}'length-(${msbROffset})) <= ${portBase}_empty;"
				} else {
					puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset})) <= ${portBase}_empty;"
				}
				append msbLOffset "+[get_parameter_value ${intfNameUpper}_NUMEMPTYBITS]" 
			}
         if { [get_parameter_value ${intfNameUpper}_SOPEOPBITSONSTREAM] != 0} {
            if { [get_parameter_value ${intfNameUpper}_DISABLESOPEOPBITSONAVST] == 0} {
					append msbROffset "+1"
					puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset})) <= ${portBase}_endofpacket;"
					append msbLOffset "+1" 
					append msbROffset "+1"
					puts $instanceOutFile "   ${dataSig}(${dataSig}'length-(${msbLOffset})) <= ${portBase}_startofpacket;"
					append msbLOffset "+1" 
				}
         }
			if { [get_parameter_value ${intfNameUpper}_REVERSESYMBOLSONAVST] == 0} {
				puts $instanceOutFile "   ${dataSig}(${portBase}_data'length-1 downto 0) <= ${portBase}_data;"
			} else {
				set symPerBeat [get_parameter_value ${intfNameUpper}_SYMBOLSPERBEAT]
				set bitsPerSym [get_parameter_value ${intfNameUpper}_DATABITSPERSYMBOL]
				
				puts $instanceOutFile "   -- reverse the symbols"
				puts $instanceOutFile "   ${portBase}_data_gen: for I in 0 to ${symPerBeat}-1 generate"
				puts $instanceOutFile "      ${dataSig}(((I+1)*${bitsPerSym}-1) downto (I*${bitsPerSym})) <= ${portBase}_data((((${symPerBeat}-I)*${bitsPerSym})-1) downto ((${symPerBeat}-I-1)*${bitsPerSym}));"
				puts $instanceOutFile "   end generate ${portBase}_data_gen;"
         }
		} else {
			set dir "out"
			set dataSig "${portBase}_data_${dir}"
			set msbLOffset "1"
			set msbROffset "0"
			if { [get_parameter_value ${intfNameUpper}_ERRORBITSONSTREAM] != 0} {
				append msbROffset "+[get_parameter_value ${intfNameUpper}_NUMERRORBITS]"
				if { [get_parameter_value ${intfNameUpper}_NUMERRORBITS] > 1} {
				puts $instanceOutFile "   ${portBase}_error <= ${dataSig}(${dataSig}'length-(${msbLOffset}) downto ${dataSig}'length-(${msbROffset}));"
				} else {
					puts $instanceOutFile "   ${portBase}_error <= ${dataSig}(${dataSig}'length-(${msbLOffset}));"
				}
				append msbLOffset "+[get_parameter_value ${intfNameUpper}_NUMERRORBITS]" 
			}
			if { [get_parameter_value ${intfNameUpper}_EMPTYBITSONSTREAM] != 0} {
				append msbROffset "+[get_parameter_value ${intfNameUpper}_NUMEMPTYBITS]"
				if { [get_parameter_value ${intfNameUpper}_NUMEMPTYBITS] > 1} {
				puts $instanceOutFile "   ${portBase}_empty <= ${dataSig}(${dataSig}'length-(${msbLOffset}) downto ${dataSig}'length-(${msbROffset}));"
				} else {
					puts $instanceOutFile "   ${portBase}_empty <= ${dataSig}(${dataSig}'length-(${msbLOffset}));"
				}
				append msbLOffset "+[get_parameter_value ${intfNameUpper}_NUMEMPTYBITS]" 
			}
         if { [get_parameter_value ${intfNameUpper}_SOPEOPBITSONSTREAM] != 0} {
				if { [get_parameter_value ${intfNameUpper}_DISABLESOPEOPBITSONAVST] == 0} {
					append msbROffset "+1"
					puts $instanceOutFile "   ${portBase}_endofpacket <= ${dataSig}(${dataSig}'length-(${msbLOffset}));"
					append msbLOffset "+1" 
					append msbROffset "+1"
					puts $instanceOutFile "   ${portBase}_startofpacket <= ${dataSig}(${dataSig}'length-(${msbLOffset}));"
					append msbLOffset "+1" 
				}
         }
			if { [get_parameter_value ${intfNameUpper}_REVERSESYMBOLSONAVST] == 0} {
			puts $instanceOutFile "   ${portBase}_data <= ${dataSig}(${portBase}_data'length-1 downto 0);"
			} else {
				set symPerBeat [get_parameter_value ${intfNameUpper}_SYMBOLSPERBEAT]
				set bitsPerSym [get_parameter_value ${intfNameUpper}_DATABITSPERSYMBOL]

				puts $instanceOutFile "   -- reverse the symbols"
				puts $instanceOutFile "   ${portBase}_data_gen: for I in 0 to [get_parameter_value ${intfNameUpper}_SYMBOLSPERBEAT]-1 generate"
				puts $instanceOutFile "      ${portBase}_data(((I+1)*${bitsPerSym}-1) downto (I*${bitsPerSym})) <= ${dataSig}((((${symPerBeat}-I)*${bitsPerSym})-1) downto ((${symPerBeat}-I-1)*${bitsPerSym}));"
				puts $instanceOutFile "   end generate ${portBase}_data_gen;"
			}
		}
				
	}
}

puts $instanceOutFile "
   -- Terminate output signals (inputs are ignored)"
foreach termPortLine $portTermPortLines {
	puts $instanceOutFile "   $termPortLine"
}
puts $instanceOutFile "
   ants_trigger_arch_module0: ants_trigger_arch_module
		generic map (
			INCLUDE_HEADER_BLOCK => INCLUDE_HEADER_BLOCK
		)
		port map (
			reset => reset,
			clk => clk,

			-- Avalon-ST Packet Connection mac_in_strm_port for bustype avalon_st_raw
			mac_in_strm_port_valid => mac_in_strm_port_valid,
			mac_in_strm_port_ready => mac_in_strm_port_ready,
			mac_in_strm_port_data => mac_in_strm_port_data_in,

			-- Avalon-ST Packet Connection mac_out_strm_port for bustype avalon_st_raw
			mac_out_strm_port_valid => mac_out_strm_port_valid,
			mac_out_strm_port_ready => mac_out_strm_port_ready,
			mac_out_strm_port_data => mac_out_strm_port_data_out,

			-- Avalon-ST Packet Connection command_strm_port for bustype avalon_st_raw
			command_strm_port_valid => command_strm_port_valid,
			command_strm_port_ready => command_strm_port_ready,
			command_strm_port_data => command_strm_port_data_in,

			-- Avalon-ST Packet Connection status_strm_port for bustype avalon_st_raw
			status_strm_port_valid => status_strm_port_valid,
			status_strm_port_ready => status_strm_port_ready,
			status_strm_port_data => status_strm_port_data_out,

			-- Avalon-ST Packet Connection trigger_strm_port for bustype avalon_st_raw
			trigger_strm_port_valid => trigger_strm_port_valid,
			trigger_strm_port_ready => trigger_strm_port_ready,
			trigger_strm_port_data => trigger_strm_port_data_out,
"
		  if {$includeHeaderBlock != 0} {
puts $instanceOutFile "
			-- header_block slave port
			header_block_ce => header_block_ce,
			header_block_read_n => header_block_read_n,
			header_block_write_n => header_block_write_n,
			header_block_addr => header_block_addr,
			header_block_wdata => header_block_wdata,
			header_block_rdata => header_block_rdata
"
		  } else {
puts $instanceOutFile "		  
    		-- header_block slave port
			header_block_addr             => (others=>'0'),
			header_block_write_n          => '0',
			header_block_read_n           => '0',
			header_block_wdata            => (others=>'0'),
			header_block_rdata            => open,
			header_block_ce               => '0'
"
		  }
puts $instanceOutFile "
		);
"

if { $useInternalCoMemoryConfig == 1 } {
	puts $instanceOutFile "
	-- internal co_memory configuration logic
	config_read_n <= '1';  -- unused
	config_addr   <= \"00\"; -- data address
	-- config_rdata (unused)
	
	process (reset,clk)
	begin
		if (reset = '1') then
			config_state(0) <= '1';
			config_state(CFG_STATE_END downto 1) <= (others=>'0');
			config_write_n <= '1';
			config_ce <= '0';
		elsif (clk'event and clk='1') then
			if (config_state(CFG_STATE_END) = '0') then
				config_state <= config_state(CFG_STATE_END-1 downto 0) & '0';
			end if;
			if (config_state(CFG_STATE_START-1) = '1') then
				config_write_n <= '0';
				config_ce <= '1';
			elsif (config_state(CFG_STATE_END-1) = '1') then
				config_write_n <= '1';
				config_ce <= '0';
			end if;
		end if;
	end process;
"
	
	puts $instanceOutFile "	config_wdata(31 downto 0) <= "
	for {set pc [expr [llength $internalCoMemoryBaseList] - 1]} {$pc >= 0} {set pc [expr {$pc - 1}]} {
		if { $pc != 0} {	
			puts $instanceOutFile "	[lindex $internalCoMemoryBaseList $pc] when config_state(CFG_STATE_START + $pc) = '1' else"
		} else {
			puts $instanceOutFile "	[lindex $internalCoMemoryBaseList $pc];"
		}
	}
}

puts $instanceOutFile "
end;
"
