// --------------------------------------------------------------------------------
// Sorrento FDK Delegated Send "Reference Design
// --------------------------------------------------------------------------------
// `timescale 1ns / 100ps
// ---------------------------------------------------------------
// ---------------------------------------------------------------
//     
// ---------------------------------------------------------------
// ---------------------------------------------------------------



module sf_fdk_ds_ref_design_core  
(
 input logic                   clk,
 input logic                   reset_n,
// -----------------------------------------------------------------------------
// Avalon MM register access bus from board_services...
// -----------------------------------------------------------------------------
 input  logic [29:0]          av_mm_slave_addrs,
 output logic [31:0]          av_mm_slave_rddata,
 input                        av_mm_slave_rd,
 input                        av_mm_slave_wr,
 input  [31:0]                av_mm_slave_wrdata,
 output logic                 av_mm_slave_wait_req,
 output logic                 av_mm_slave_wait_rd_vld,
 
// -----------------------------------------------------------------------------
// Streaming packet interface from wire side... Rx inputs :1x10G Only
// -----------------------------------------------------------------------------

 input  logic                i_wire_link_up,
 input  logic                i_wire_valid,
 input  logic                i_wire_sop,
 input  logic                i_wire_eop,
 input  logic[255:0]         i_wire_data,
 input  logic[4:0]           i_wire_eop_byte_count, 
 output logic                o_wire_ready,
// -----------------------------------------------------------------------------   
// Streaming packet interface to wire side... Tx outputs  :1x10G Only                          
// -----------------------------------------------------------------------------    
                                                                                    
 output logic                o_wire_valid,               
 output logic                o_wire_sop,                 
 output logic                o_wire_eop,                 
 output logic[255:0]         o_wire_data,                
 output logic[4:0]           o_wire_eop_byte_count,      
 input  logic                i_wire_ready,               
                                                                                    

// -----------------------------------------------------------------------------
// Tx data interface out to NIC side Tx MACs... Rx outputs :1x10G Only 

// -----------------------------------------------------------------------------

 output logic                 o_nic_valid,
 output logic                 o_nic_sop,
 output logic                 o_nic_eop,
 output logic[255:0]          o_nic_data,
 output logic[4:0]            o_nic_eop_byte_count, 
 input  logic                 i_nic_ready,
 
// -----------------------------------------------------------------------------
// Streaming packet interface from NIC side... Tx inputs
// -----------------------------------------------------------------------------

 input logic                  i_nic_link_up,
 input logic                  i_nic_valid,
 input logic                  i_nic_sop,
 input logic                  i_nic_eop,
 input logic[255:0]           i_nic_data,
 input logic[4:0]             i_nic_eop_byte_count, 
 output logic                 o_nic_ready,
// -----------------------------------------------------------------------------
// Time_Stamping : For Debugging
// -----------------------------------------------------------------------------
 
 
 // Rx packet timestamp interface from MACs:

 input [63:0]                 rx_packet_timestamp,
 input                        rx_packet_timestamp_valid,
 // Tx packet timestamp interface from MACs:
 input                        i_wire_sys_ptp_tx_2step_en,
 input [63:0]                 i_wire_sys_ptp_tx_2step_dat
   
 );
  


 
  
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 
//  Signals                                                                
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 

logic[31:0]    cpu_addr;
logic[31:0]    cpu_wrdata;
logic[31:0]    cpu_rddata;
logic          cpu_cs;
logic          cpu_rd;
logic          cpu_wr;
logic          cpu_wr_ack;
logic          cpu_rd_valid;
//cs 
logic          fdk_rsvd_cs;
logic          fdk_dbg_cs;
logic          fdk_tx_cs;
logic          fdk_rx_cs;
//rd data
logic[31:0]    fdk_rx_rddata;
logic[31:0]    fdk_tx_rddata;
logic[31:0]    fdk_dbg_rddata;
//rd vld
logic          fdk_rx_rd_vld;
logic          fdk_tx_rd_vld;
logic          fdk_dbg_rd_vld;
//wr_ack
logic          fdk_rx_wr_ack;
logic          fdk_tx_wr_ack;
logic          fdk_dbg_wr_ack;
//wire to nic
logic[255:0]   algo_data_out;
logic[4:0]     algo_empty_out;
logic          algo_ready_out;
logic          algo_sop_out;
logic          algo_eop_out;
logic          algo_vld_out;

logic[255:0]   ants_host_data_out;
logic[4:0]     ants_host_eop_empty_out;
logic          ants_host_rdy_out;
logic          ants_host_sop_out;
logic          ants_host_eop_out;
logic          ants_host_vld_out;

logic          ants_gen_pkt_to_nic_tx_en;
logic          wire_gen_pkt_to_nic_tx_en;
//algo
logic[7:0]     algo_msg_id_out;
logic[7:0]     algo_conn_id_out;
logic[31:0]    algo_msg_info_out;
logic          algo_trg_vld_out;
//nic_to_wire
logic          ants_gen_pkt_to_wire_tx_en;
logic          nic_gen_pkt_to_wire_tx_en;
logic          algo_trg_vld_out_tmp;
logic          dbg_algo;
logic          dbg_algo_trig;
logic          dbg_algo_trig_pulse;
logic          dbg_total_latency_en;
//dbg   
logic[4:0]    dbg_cntr_en;   
logic[7:0]    dbg_gp_out;
  

//---------------------------------------------------------------------------------------------- 
//----------------------------------------------------------------------------------------------  
// Impulse C system
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 
  
logic[255:0] from_mac_data;
logic from_mac_valid;
logic from_mac_ready;
logic from_mac_sop;
logic from_mac_eop;
logic[4:0] from_mac_empty;

logic[255:0] impc_passthrough_data;
logic impc_passthrough_valid;
logic impc_passthrough_ready;
logic impc_passthrough_sop;
logic impc_passthrough_eop;
logic[4:0] impc_passthrough_empty;

logic trigger_strm_valid;
logic[47:0] trigger_strm_data;


assign from_mac_data = i_wire_data;
assign from_mac_valid = i_wire_valid;
assign o_wire_ready = from_mac_ready;
assign from_mac_sop = i_wire_sop;
assign from_mac_eop = i_wire_eop;
//assign from_mac_empty = i_wire_eop_byte_count;
assign from_mac_empty = ((5'd31 - i_wire_eop_byte_count)); // Tamba_valid_eop_byte to empty eop empty bytes


logic [31:0] cmd_strm_port_data;
logic cmd_strm_port_valid;
logic [31:0] stat_strm_port_data;
logic stat_strm_port_ready;
logic stat_strm_port_valid;
logic fdk_rsvd_ack;

assign cmd_strm_port_data = cpu_wrdata;

always_ff @ (posedge clk or negedge reset_n) begin
	if (!reset_n) begin
	  cmd_strm_port_valid <=1'b0;
	  stat_strm_port_ready <=1'b0;
	  fdk_rsvd_ack <=1'b0;
end else begin
		// single pulse on valid
		cmd_strm_port_valid <= (fdk_rsvd_cs & cpu_wr & !cmd_strm_port_valid & !fdk_rsvd_ack);
		stat_strm_port_ready <= (fdk_rsvd_cs & cpu_rd & !stat_strm_port_ready & !fdk_rsvd_ack);
		fdk_rsvd_ack <= (cmd_strm_port_valid | stat_strm_port_ready | fdk_rsvd_ack) 
								& (fdk_rsvd_cs & ( cpu_wr | cpu_rd));
	end
end  


  impc_qsys impc_qsys_inst (
        .clk_clk                  (clk),                  //        clk.clk
        .reset_reset_n            (reset_n),            //      reset.reset_n
//        .in_stream_data           (from_mac_data),           //  in_stream.data
//        .in_stream_valid          (from_mac_valid),          //           .valid
//        .in_stream_ready          (from_mac_ready),          //           .ready
//        .in_stream_startofpacket  (from_mac_sop),  //           .startofpacket
//        .in_stream_endofpacket    (from_mac_eop),    //           .endofpacket
//        .in_stream_empty          (from_mac_empty),          //           .empty
//        .out_stream_data          (impc_passthrough_data),          // out_stream.data
//        .out_stream_valid         (impc_passthrough_valid),         //           .valid
//        .out_stream_ready         (impc_passthrough_ready),         //           .ready
//        .out_stream_startofpacket (impc_passthrough_sop), //           .startofpacket
//        .out_stream_endofpacket   (impc_passthrough_eop),   //           .endofpacket
//        .out_stream_empty         (impc_passthrough_empty)          //           .empty

        .command_strm_port_ready         (),
        .command_strm_port_valid         (cmd_strm_port_valid),//(1'b0),
        .command_strm_port_data          (cmd_strm_port_data),//(32'd0),
        .command_strm_port_endofpacket   (1'b0),
        .command_strm_port_startofpacket (1'b0),
        .status_strm_port_ready          (stat_strm_port_ready),
        .status_strm_port_valid          (stat_strm_port_valid),
        .status_strm_port_data           (stat_strm_port_data),//(32'd0),
        .status_strm_port_endofpacket    (1'b0),
        .status_strm_port_startofpacket  (1'b0),
        .mac_in_strm_port_ready          (from_mac_ready),
        .mac_in_strm_port_valid          (from_mac_valid),
        .mac_in_strm_port_data           (from_mac_data),
        .mac_in_strm_port_endofpacket    (from_mac_eop),
        .mac_in_strm_port_startofpacket  (from_mac_sop),
        .mac_in_strm_port_empty          (from_mac_empty),
        .mac_out_strm_port_ready         (impc_passthrough_ready),
        .mac_out_strm_port_valid         (impc_passthrough_valid),
        .mac_out_strm_port_data          (impc_passthrough_data),
        .mac_out_strm_port_endofpacket   (impc_passthrough_eop),
        .mac_out_strm_port_startofpacket (impc_passthrough_sop),
        .mac_out_strm_port_empty         (impc_passthrough_empty),
        .trigger_strm_port_ready         (1'b1),
        .trigger_strm_port_valid         (trigger_strm_valid),
        .trigger_strm_port_data          (trigger_strm_data),
        .trigger_strm_port_endofpacket   (),
        .trigger_strm_port_startofpacket ()
    );
  
  
//---------------------------------------------------------------------------------------------- 
//----------------------------------------------------------------------------------------------  
//FDK_Rx ( Wire to NIC )Datapath
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 
  sf_ds_wire_to_nic 
   sf_ds_wire_to_nic_inst                          
   (                                                                                        
  //Reset & Clk                         
  .reset_n                              (reset_n),
  .clk                                  (clk),
  //interface Frm Wire MAC_Rx (in)      
  .mac_rx_wire_ready_out                (impc_passthrough_ready),
  .mac_rx_wire_vld_in                   (impc_passthrough_valid),
  .mac_rx_wire_sop_in                   (impc_passthrough_sop),
  .mac_rx_wire_eop_in                   (impc_passthrough_eop),
  .mac_rx_wire_data_in                  (impc_passthrough_data),
  .mac_rx_wire_eop_vld_bytes_in         ((5'd31 - impc_passthrough_empty) + 5'd1), // Tamba_valid_eop_byte to empty eop empty bytes
//  .mac_rx_wire_ready_out                (o_wire_ready),
//  .mac_rx_wire_vld_in                   (i_wire_valid),
//  .mac_rx_wire_sop_in                   (i_wire_sop),
//  .mac_rx_wire_eop_in                   (i_wire_eop),
//  .mac_rx_wire_data_in                  (i_wire_data),
//  .mac_rx_wire_eop_vld_bytes_in         (i_wire_eop_byte_count),
  .mac_rx_wire_link_up_in               (i_wire_link_up),
  //interface to NIC MAC_Tx (out)       
  .mac_tx_nic_ready_in                  (i_nic_ready),
  .mac_tx_nic_vld_out                   (o_nic_valid),
  .mac_tx_nic_sop_out                   (o_nic_sop),
  .mac_tx_nic_eop_out                   (o_nic_eop),
  .mac_tx_nic_eop_vld_bytes_out         (o_nic_eop_byte_count),
  .mac_tx_nic_data_out                  (o_nic_data),
  //interface from ANTS (in)            
  .ants_host_rdy_out                    (ants_host_rdy_out),
  .ants_host_vld_in                     (ants_host_vld_out),
  .ants_host_sop_in                     (ants_host_sop_out),
  .ants_host_eop_in                     (ants_host_eop_out),
  .ants_host_data_in                    (ants_host_data_out),
  .ants_host_eop_empty_in               (ants_host_eop_empty_out),
  //interface to Algo(out)              
  .algo_rdy_in                          (algo_ready_out),
  .algo_vld_out                         (algo_vld_out),
  .algo_sop_out                         (algo_sop_out),
  .algo_eop_out                         (algo_eop_out),
  .algo_empty_out                       (algo_empty_out),
  .algo_data_out                        (algo_data_out),
  //CPU Interface                       
  .cpu_cs                               (fdk_rx_cs),
  .cpu_wr                               (fdk_rx_cs & cpu_wr ),
  .cpu_rd                               (fdk_rx_cs & cpu_rd ),
  .cpu_wr_data                          (cpu_wrdata),
  .cpu_addr                             (cpu_addr),
  .cpu_rd_data                          (fdk_rx_rddata),
  .cpu_wr_ack                           (fdk_rx_wr_ack),
  .cpu_rd_vld                           (fdk_rx_rd_vld),
   //dbg counters
  .ants_gen_pkt_to_nic_tx_en           (ants_gen_pkt_to_nic_tx_en),
  .wire_gen_pkt_to_nic_tx_en           (wire_gen_pkt_to_nic_tx_en)
   );

//----------------------------------------------------------------------------------------------    
//----------------------------------------------------------------------------------------------    
//Simple Tick to Trade Algo                                                                    
//----------------------------------------------------------------------------------------------    
//----------------------------------------------------------------------------------------------    
 
 //String is "hit me " and the hex vaule is h=68, i=69 , t=74,space=20, m= 6D, e=65, space= 20 
// Use trigger output from impc_qsys
assign algo_ready_out = 1'b1;
assign algo_msg_id_out = trigger_strm_data[47:40];
assign algo_conn_id_out = trigger_strm_data[39:32];
assign algo_msg_info_out = trigger_strm_data[31:0];
assign algo_trg_vld_out_tmp = trigger_strm_valid;

//sf_ds_sample_algo #
//   (
//   .MSG_BYTE_PER_CYCLE       (32),//     
//   .MATCH_NUM_OF_BYTE        (7), // How many bytes to be searched in the msg. <30
//   .MATCH_START_OFFSET       (0), // Start offset of Match w.r.t. the SOP 
//   .CONN_ID_OFFSET           (7), // Offset of Connection ID w.r.t. the SOP   
//   .MSG_ID_OFFSET            (9),  // Offset of  Msg ID w.r.t. the SOP 
//   .SHORT_MSG_EOP_EMTPTY_VAL (5'd23), // >
//   .PIPE_EN                  (0)
//    )
//  sf_ds_sample_algo_inst
// (
//  //Reset & Clk         
//  .reset_n              (reset_n),
//  .clk                  (clk),
//  //input Interface                    
//  .ready_out            (algo_ready_out),
//  .in_vld               (algo_vld_out),
//  .in_sop               (algo_sop_out),
//  .in_eop               (algo_eop_out),
//  .in_data              (algo_data_out),
//  .in_eop_pos           (algo_empty_out),
//  .in_ref_value         (56'h686974206D6520),
//  .msg_id_out           (algo_msg_id_out),
//  .conn_id_out          (algo_conn_id_out),
//  .msg_info_out         (algo_msg_info_out),
//  .algo_trg_vld         (algo_trg_vld_out_tmp)
// ); 
 //dbg_algo_trigger
always_ff @ (posedge clk or negedge reset_n) begin

if (!reset_n) begin
  dbg_algo                <= 1'd0;
  dbg_algo_trig            <= 1'd0;
  dbg_total_latency_en    <=1'd0;

end else begin
  dbg_algo                <= dbg_gp_out[1];
  dbg_algo_trig           <= dbg_algo;
  if (dbg_total_latency_en & algo_sop_out & algo_ready_out & algo_vld_out) //end
      dbg_total_latency_en <=1'd0;
  else if (algo_trg_vld_out)
      dbg_total_latency_en <=1'd1; //start
      
  end
end  
 
 assign dbg_algo_trig_pulse = dbg_algo & ~ dbg_algo_trig;
 assign algo_trg_vld_out    = dbg_gp_out[2] ? dbg_algo_trig_pulse : algo_trg_vld_out_tmp;
   
//---------------------------------------------------------------------------------------------- 
//----------------------------------------------------------------------------------------------  
//FDK_Tx ( NIC to Wire )Datapath
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 

 sf_ds_nic_to_wire 
  sf_ds_nic_to_wire_inst                           
   (                                                                                        
  //Reset & Clk  
  .reset_n                                (reset_n),
  .clk                                    (clk),
  //interface frm Algo(in)                
  .msg_id_in                              (algo_msg_id_out),
  .conn_id_in                             (algo_conn_id_out),
  .msg_info_in                            (algo_msg_info_out),
  .algo_trg_in                            (algo_trg_vld_out),
  //interface frm NIC MAC_Rx (in)         
  .mac_rx_nic_rdy_out                     (o_nic_ready),
  .mac_rx_nic_vld_in                      (i_nic_valid),
  .mac_rx_nic_sop_in                      (i_nic_sop),
  .mac_rx_nic_eop_in                      (i_nic_eop),
  .mac_rx_nic_data_in                     (i_nic_data),
  .mac_rx_nic_eop_vld_bytes_in            (i_nic_eop_byte_count),
  .mac_rx_nic_link_up_in                  (i_nic_link_up),
  //interface to Wire MAC_Tx (out)        
  .mac_tx_wire_ready_in                   (i_wire_ready),
  .mac_tx_wire_vld_out                    (o_wire_valid),
  .mac_tx_wire_sop_out                    (o_wire_sop),
  .mac_tx_wire_eop_out                    (o_wire_eop),
  .mac_tx_wire_eop_valid_bytes_out        (o_wire_eop_byte_count),
  .mac_tx_wire_data_out                   (o_wire_data),
  //interface to host (insert_prefix)     
  .ants_host_rdy_in                       (ants_host_rdy_out),
  .ants_host_vld_out                      (ants_host_vld_out),
  .ants_host_sop_out                      (ants_host_sop_out),
  .ants_host_eop_out                      (ants_host_eop_out),
  .ants_host_data_out                     (ants_host_data_out),
  .ants_host_eop_empty_out                (ants_host_eop_empty_out),
  //CPU Interface                       
  .cpu_cs                                 (fdk_tx_cs),
  .cpu_wr                                 (fdk_tx_cs & cpu_wr ),
  .cpu_rd                                 (fdk_tx_cs & cpu_rd ),
  .cpu_wr_data                            (cpu_wrdata),
  .cpu_addr                               (cpu_addr),
  .cpu_rd_data                            (fdk_tx_rddata),
  .cpu_wr_ack                             (fdk_tx_wr_ack),
  .cpu_rd_vld                             (fdk_tx_rd_vld),
  //dbg counters                          
  .ants_gen_pkt_to_wire_tx_en             (ants_gen_pkt_to_wire_tx_en),
  .nic_gen_pkt_to_wire_tx_en              (nic_gen_pkt_to_wire_tx_en) 

   ); 
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 
//  Avalon MM to CSR convertion   ;Control Path                                                          
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 

  sf_ssc_av_mm_slave_adapter # (.ADR_WIDTH (32),.ADDRS_CMPR_MSB_NIBBLE(4'h4),.ADDRS_MS_NIBBLE_CMPR_ON (1'b0))
  sf_ssc_av_mm_slave_adapter_inst
 (
 .reset_n           (reset_n),
 .clk               (clk),
 .avalon_addr       ({2'b01,av_mm_slave_addrs}),                                                                        
 .avalon_wrdata     (av_mm_slave_wrdata),                                                                           
 .avalon_rd         (av_mm_slave_rd),                                                                  
 .avalon_wr         (av_mm_slave_wr),
 .avalon_rddata     (av_mm_slave_rddata),                                                                           
 .avalon_wait_req   (av_mm_slave_wait_req),
 .avalon_rd_valid   (av_mm_slave_wait_rd_vld),
 .cpu_cs            (cpu_cs),                                                            
 .cpu_addr          (cpu_addr),                                                                         
 .cpu_wrdata        (cpu_wrdata),                                                                      
 .cpu_rd            (cpu_rd),                                                                
 .cpu_wr            (cpu_wr),
 .cpu_rddata        (cpu_rddata),                                                                      
 .cpu_wr_ack        (cpu_wr_ack),
 .cpu_rd_valid      (cpu_rd_valid)
 ); 

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//  Chip Select decoding                                                                 
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

// =============================================================================================
// Decoding the input Avalon-MM ffrom board_services and fanning out to the following:                            
// 0x4XXX_XXXX : FDK Rx;                                                                                          
// 0x5XXX_XXXX : FDK Tx;                                                                                          
// 0x6XXX_XXXX : FDK_Dbg;  for Debug                                                                
// 0x7XXX_XXXX : Stream in/out of impc_qsys;                                             
// ===============================================================================================

assign {fdk_rsvd_cs,fdk_dbg_cs,fdk_tx_cs,fdk_rx_cs}= (cpu_addr[31:28]== 4'b0100)? {3'b0,cpu_cs}      : 
                                                     (cpu_addr[31:28]== 4'b0101)? {2'b0,cpu_cs,1'b0} :
                                                     (cpu_addr[31:28]== 4'b0110)? {1'b0,cpu_cs,2'b0} :
                                                     (cpu_addr[31:28]== 4'b0111)? {cpu_cs,3'b0} : 4'b0;
                                                      

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//  CPU Read Data Muxing                                                               
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
assign cpu_rddata         = (cpu_addr[29:28]== 2'b00)?  fdk_rx_rddata :
                            (cpu_addr[29:28]== 2'b01)?  fdk_tx_rddata : 
                            (cpu_addr[29:28]== 2'b10)?  fdk_dbg_rddata : stat_strm_port_data;//32'h0; 

                            
                            
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
// CPU Wr Ack Muxing                                                               
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
assign cpu_wr_ack         = (cpu_addr[29:28]== 2'b00)?  fdk_rx_wr_ack :
                            (cpu_addr[29:28]== 2'b01)?  fdk_tx_wr_ack : 
                            (cpu_addr[29:28]== 2'b10)?  fdk_dbg_wr_ack : 
                            (fdk_rsvd_ack & fdk_rsvd_cs); // 1'b0; 
                                                        
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
// CPU Read Valid Muxing                                                               
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
assign cpu_rd_valid         = (cpu_addr[29:28]== 2'b00)?  fdk_rx_rd_vld :
                              (cpu_addr[29:28]== 2'b01)?  fdk_tx_rd_vld : 
                              (cpu_addr[29:28]== 2'b10)?  fdk_dbg_rd_vld : stat_strm_port_valid;//1'b0; 
                            
                                                        

//---------------------------------------------------------------------------------------------- 
//----------------------------------------------------------------------------------------------  
//Debug Hooks
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 
 sf_general_purpose_regs 
 sf_general_purpose_regs_inst
(   
  //CPU Interface
  .cpu_clk              (clk),
  .cpu_reset            (~reset_n),
  .cpu_wr               (cpu_wr),
  .cpu_cs               (fdk_dbg_cs),
  .cpu_rd               (cpu_rd),
  .cpu_wrdata           (cpu_wrdata),
  .cpu_addr             (cpu_addr[6:0]),
  .cpu_rddata           (fdk_dbg_rddata),
  .cpu_rd_vld           (fdk_dbg_rd_vld),
  .cpu_wr_ack           (fdk_dbg_wr_ack),
  .dbg_gp_out           (dbg_gp_out),
  .dbg_gp_in            (32'hFEEDBABE),
  .dbg_cntr_en          (dbg_cntr_en),
  //Time-Stamping       
  .mac_tx_ts_en         (i_wire_sys_ptp_tx_2step_en),
  .mac_tx_ts_val        (i_wire_sys_ptp_tx_2step_dat),
  .mac_rx_ts_en         (rx_packet_timestamp_valid),
  .mac_rx_ts_val        (rx_packet_timestamp),
  //Local Time-Stamping
  .mac_wire_rx_vld_sop  (i_wire_valid & i_wire_sop) ,
  .mac_wire_tx_vld_sop  (o_wire_valid & o_wire_sop)

 ); 

//---------------------------------------------------------------------------------------------- 
//----------------------------------------------------------------------------------------------  
//Debug Reg Details  :
//offset 0x0 = 32bits R/W reg = dbg_gp_out
//offset 0x4 = bit[0] R/W = toggle 1->0 to clr all dbg_counters
//offset 0x8 = 32bits R/O reg =  dbg_gp_in :
//offset 0xC = 32bits R/O Dbg_Counter0 associated with dbg_cntr_en[0] pulse
//offset 0x10 = 32bits R/O Dbg_Counter1 associated with dbg_cntr_en[1] pulse
//offset 0x14 = 32bits R/O Dbg_Counter1 associated with dbg_cntr_en[2] pulse
//offset 0x18 = 32bits R/O Dbg_Counter1 associated with dbg_cntr_en[3] pulse
//offset 0x1C = 32bits R/O Dbg_Counter1 associated with dbg_cntr_en[4] pulse
//offset 0x20 = MAC_RX_SOP to MAC TX SOP Core Timing [31:0]  ;RO
//offset 0x24 = MAC_RX_TS_VAL[63:32] ;RO
//offset 0x28 = MAC_RX_TS_VAL[31:0]  ;RO
//offset 0x2c = MAC_TX_TS_VAL[63:32] ;RO
//offset 0x30 = MAC_TX_TS_VAL[31:0]  ;RO
//---------------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------------- 

//offset 0xC   = dbg_counter0 = algo_trigger_count (Rx_tick count);
assign dbg_cntr_en[0] =  algo_trg_vld_out;

//offset 0x10  = dbg_counter1 = ants_gen_pkt_to_wire_tx_en(Tx_trade count) ; 
assign dbg_cntr_en[1] =  ants_gen_pkt_to_wire_tx_en;

//offset 0x14  = dbg_counter2 = nic_gen_pkt_to_wire_tx_en(pass thrgh pkt Tx count from nic_rx to wire Tx) ; 
assign dbg_cntr_en[2] =  dbg_gp_out[4] ? dbg_total_latency_en : nic_gen_pkt_to_wire_tx_en;

//offset 0x18  = dbg_counte3 = ants_gen_pkt_to_nic_tx_en(ants generated pkt to the nic_host_tx) ; 
assign dbg_cntr_en[3] =  ants_gen_pkt_to_nic_tx_en;

//offset 0x1C  = dbg_counte4 = wire_gen_pkt_to_nic_tx_en(pass thrgh pkt Tx count wire Rx to nic_tx) ; 
assign dbg_cntr_en[4] =  wire_gen_pkt_to_nic_tx_en;
 

 
 
endmodule  








